﻿local ADDON_NAME = "iView"
local ADDON_VERSION = "17"
local API = {100024, nil}

local UNIT_TAG_PLAYER = "player"
local SWF = LibStub("LibScootworksFunctions")

local DEFAULTS =
{
	forceFirstPerson = false,
	distanceDefault = true,
	enableInCombat = true,
	enableOnMount = true,
	enableInStealth = true,
	valueDefault = 8,
	valueCombat = 5,
	valueMount = 10,
	valueStealth = 3,
	valueConversation = 0,
}

local savedVars
local localInCombat, localMounted, localInStealth, localInConversation = false, false, false, false

local function SetCameraDistance(zoom)
	SetSetting(SETTING_TYPE_CAMERA, CAMERA_SETTING_DISTANCE, zoom)
end

local function ChangeCameraZoom()
	if savedVars.enableInConversation and localInConversation then
		SetCameraDistance(savedVars.valueConversation)
		return
	end

	if savedVars.enableOnMount and localMounted then
		SetCameraDistance(savedVars.valueMount)
		return
	end	
	if savedVars.enableInStealth and localInStealth then
		SetCameraDistance(savedVars.valueStealth)
		return
	end
	if savedVars.enableInCombat and localInCombat then
		SetCameraDistance(savedVars.valueCombat)
		return
	end
	
	if localMounted or localInCombat or localInStealth then
		return
	elseif savedVars.distanceDefault then
		SetCameraDistance(savedVars.valueDefault)
	end
end

local function CheckStates()
	if not savedVars.forceFirstPerson then
		ChangeCameraZoom()
	else
		SetSetting(SETTING_TYPE_CAMERA, CAMERA_SETTING_DISTANCE, 0)
	end
end


--------------
-- SETTINGS --
--------------
local function InitializeSettingsMenu()

	local menu = LibStub("LibAddonMenu-2.0")
	
	local panel = {
		type = "panel",
		name = ADDON_NAME,
		displayName = "|cFF3DA5i|rView",
		author = "Darkretailer, Scootworks",
		version = ADDON_VERSION,
		website = "http://www.esoui.com/downloads/info722-iView.html",
		slashCommand = "/iview",
		registerForRefresh = true,
		registerForDefaults = true,
	}

	local options = {
		{
			type = "header",
			name = SI_AUDIO_OPTIONS_GENERAL,
			width = "full",
		},
		{
			type = "checkbox",
			name = SI_IVIEW_FIRST_PERSON,
			getFunc = function() return savedVars.forceFirstPerson end,
			setFunc = function(value)
				savedVars.forceFirstPerson = value
			end,
			default = DEFAULTS.forceFirstPerson
		},
		{
			type = "header",
			name = SI_OPTIONS_DEFAULTS,
		},
		{
			type = "checkbox",
			name = SI_IVIEW_DESC_DEFAULT,
			width = "half",
			getFunc = function() return savedVars.distanceDefault end,
			setFunc = function(value)
				savedVars.distanceDefault = value
			end,
			disabled = function() return savedVars.forceFirstPerson end,
			default = DEFAULTS.distanceDefault
		},
		{
			type = "slider",
			name = SI_WORLD_MAP_ZOOM,
			tooltip = SI_IVIEW_DESC_STEP,
			min = 0,
			max = 10,
			step = 0.5,
			width = "half",
			inputLocation = "right",
			getFunc = function() return savedVars.valueDefault end,
			setFunc = function(value)
				savedVars.valueDefault = value
			end,
			disabled = function() return savedVars.forceFirstPerson end,
			default = DEFAULTS.valueDefault
		},
		{
			type = "header",
			name = SI_AUDIO_OPTIONS_COMBAT,
		},
		{
			type = "checkbox",
			name = SI_IVIEW_DESC_COMBAT,
			width = "half",
			getFunc = function() return savedVars.enableInCombat end,
			setFunc = function(value)
				savedVars.enableInCombat = value
			end,
			disabled = function() return savedVars.forceFirstPerson end,
			default = DEFAULTS.enableInCombat
		},
		{
			type = "slider",
			name = SI_WORLD_MAP_ZOOM,
			tooltip = SI_IVIEW_DESC_STEP,
			min = 0,
			max = 10,
			step = 0.5,
			width = "half",
			inputLocation = "right",
			getFunc = function() return savedVars.valueCombat end,
			setFunc = function(value)
				savedVars.valueCombat = value
			end,
			disabled = function() return savedVars.forceFirstPerson end,
			default = DEFAULTS.valueCombat
		},
		{
			type = "header",
			name = SI_COLLECTIBLECATEGORYTYPE2,
		},
		{
			type = "checkbox",
			name = SI_IVIEW_DESC_MOUNT,
			width = "half",
			getFunc = function() return savedVars.enableOnMount end,
			setFunc = function(value)
				savedVars.enableOnMount = value
			end,
			disabled = function() return savedVars.forceFirstPerson end,
			default = DEFAULTS.enableOnMount
		},
		{
			type = "slider",
			name = SI_WORLD_MAP_ZOOM,
			tooltip = SI_IVIEW_DESC_STEP,
			min = 0,
			max = 10,
			step = 0.5,
			width = "half",
			inputLocation = "right",
			getFunc = function() return savedVars.valueMount end,
			setFunc = function(value)
				savedVars.valueMount = value
			end,
			disabled = function() return savedVars.forceFirstPerson end,
			default = DEFAULTS.valueMount,
			},
		{
			type = "header",
			name = zo_strformat(SI_STEALTH_HIDDEN), -- it has to be in zo_strformat, otherwise it will not work on french client
		},
		{
			type = "checkbox",
			name = SI_IVIEW_DESC_STEALTH,
			width = "half",
			getFunc = function() return savedVars.enableInStealth end,
			setFunc = function(value)
				savedVars.enableInStealth = value
			end,
			disabled = function() return savedVars.forceFirstPerson end,
			default = DEFAULTS.enableInStealth,
			},

		{
			type = "slider",
			name = SI_WORLD_MAP_ZOOM,
			tooltip = SI_IVIEW_DESC_STEP,
			min = 0,
			max = 10,
			step = 0.5,
			width = "half",
			inputLocation = "right",
			getFunc = function() return savedVars.valueStealth end,
			setFunc = function(value)
				savedVars.valueStealth = value
			end,
			disabled = function() return savedVars.forceFirstPerson end,
			default = DEFAULTS.valueStealth,
		},
		{
			type = "header",
			name = SI_IVIEW_HEADER_CONVERSATION,
		},
		{
			type = "checkbox",
			name = SI_IVIEW_DESC_CONVERSATION,
			width = "half",
			getFunc = function() return savedVars.enableInConversation end,
			setFunc = function(value)
				savedVars.enableInConversation = value
			end,
			disabled = function() return savedVars.forceFirstPerson end,
			default = DEFAULTS.enableInConversation,
			},

		{
			type = "slider",
			name = SI_WORLD_MAP_ZOOM,
			tooltip = SI_IVIEW_DESC_STEP,
			min = 0,
			max = 10,
			step = 0.5,
			width = "half",
			inputLocation = "right",
			getFunc = function() return savedVars.valueConversation end,
			setFunc = function(value)
				savedVars.valueConversation = value
			end,
			disabled = function() return savedVars.forceFirstPerson end,
			default = DEFAULTS.valueConversation,
		},
	},

	menu:RegisterAddonPanel(ADDON_NAME.."OptionsMenu", panel)
	menu:RegisterOptionControls(ADDON_NAME.."OptionsMenu", options)

end

----------
-- INIT --
----------
local function OnAddOnLoaded(_, addOnName)
	if addOnName == ADDON_NAME then
		--if not SWF:IsAddonVersionSameGameVersion(API[1], API[2], ADDON_NAME) then return end

		local TRACKED_STEALTH_STATES =
		{
			STEALTH_STATE_DETECTED,
			STEALTH_STATE_HIDDEN_ALMOST_DETECTED,
			STEALTH_STATE_HIDDEN,
			STEALTH_STATE_STEALTH_ALMOST_DETECTED,
			STEALTH_STATE_STEALTH,
		}

		savedVars = ZO_SavedVars:NewAccountWide(ADDON_NAME.."_Save", 1, nil, DEFAULTS)
		InitializeSettingsMenu()
		
		local function IsStealthed(stealthState)
			local playerStealthState = stealthState
			if playerStealthState == nil then
				playerStealthState = GetUnitStealthState(UNIT_TAG_PLAYER)
			end
			local state = TRACKED_STEALTH_STATES[playerStealthState]
			if state == nil then
				return false
			end
			return true
		end
		
		localInCombat = IsUnitInCombat(UNIT_TAG_PLAYER)
		localMounted = IsMounted()
		localInStealth = IsStealthed()
		ChangeCameraZoom()

		EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_MOUNTED_STATE_CHANGED, function(_, mounted)
			if localMounted ~= mounted then
				localMounted = mounted
				CheckStates()
			end
		end)
		
		EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_STEALTH_STATE_CHANGED, function(_, _, stealthState)
			local stealthed = IsStealthed(stealthState)
			if localInStealth ~= stealthed then
				localInStealth = stealthed
				CheckStates()
			end
		end)
		
		EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_PLAYER_COMBAT_STATE, function(_, inCombat)
			if localInCombat ~= inCombat then
				localInCombat = inCombat
				CheckStates()
			end
		end)
		
        EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_CHATTER_BEGIN, function()
			if not localInConversation then
				localInConversation = true
				CheckStates()
			end
        end)
        
        EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_CHATTER_END, function()
			if localInConversation then
				localInConversation = false
				CheckStates()
			end
        end)
		
		EVENT_MANAGER:AddFilterForEvent(ADDON_NAME, EVENT_MOUNTED_STATE_CHANGED, REGISTER_FILTER_UNIT_TAG, UNIT_TAG_PLAYER)
		EVENT_MANAGER:AddFilterForEvent(ADDON_NAME, EVENT_STEALTH_STATE_CHANGED, REGISTER_FILTER_UNIT_TAG, UNIT_TAG_PLAYER)
		EVENT_MANAGER:AddFilterForEvent(ADDON_NAME, EVENT_PLAYER_COMBAT_STATE, REGISTER_FILTER_UNIT_TAG, UNIT_TAG_PLAYER)
	end
end

EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_ADD_ON_LOADED, OnAddOnLoaded)